@@ -1,69 +1,13 @@
-#include <ruby.h>
-#include <string.h>
-#include <math.h>
-#include <time.h>
-#include <locale.h>
-#include <sqlite3.h>
-#include "compat.h"
+#include "do_sqlite3.h"
 #include "error.h"
-
-#ifdef HAVE_RUBY_ENCODING_H
-#include <ruby/encoding.h>
-
-#define DO_STR_NEW2(str, encoding) \
-  ({ \
-    VALUE _string = rb_str_new2((const char *)str); \
-    if(encoding != -1) { \
-      rb_enc_associate_index(_string, encoding); \
-    } \
-    _string; \
-  })
-
-#define DO_STR_NEW(str, len, encoding) \
-  ({ \
-    VALUE _string = rb_str_new((const char *)str, (long)len); \
-    if(encoding != -1) { \
-      rb_enc_associate_index(_string, encoding); \
-    } \
-    _string; \
-  })
-
-#else
-
-#define DO_STR_NEW2(str, encoding) \
-  rb_str_new2((const char *)str)
-
-#define DO_STR_NEW(str, len, encoding) \
-  rb_str_new((const char *)str, (long)len)
-#endif
-
-
-#define ID_CONST_GET rb_intern("const_get")
-#define ID_PATH rb_intern("path")
-#define ID_NEW rb_intern("new")
-#define ID_ESCAPE rb_intern("escape_sql")
-#define ID_QUERY rb_intern("query")
-
-#define RUBY_CLASS(name) rb_const_get(rb_cObject, rb_intern(name))
-#define CONST_GET(scope, constant) (rb_funcall(scope, ID_CONST_GET, 1, rb_str_new2(constant)))
-#define SQLITE3_CLASS(klass, parent) (rb_define_class_under(mSqlite3, klass, parent))
-
-#ifdef _WIN32
-#define do_int64 signed __int64
-#else
-#define do_int64 signed long long int
-#endif
-
-#ifndef HAVE_SQLITE3_PREPARE_V2
-#define sqlite3_prepare_v2 sqlite3_prepare
-#endif
-
 // To store rb_intern values
+
+static ID ID_NEW;
 static ID ID_NEW_DATE;
+static ID ID_CONST_GET;
 static ID ID_RATIONAL;
-static ID ID_LOGGER;
-static ID ID_DEBUG;
-static ID ID_LEVEL;
+static ID ID_ESCAPE;
+static ID ID_LOG;
 
 static VALUE mExtlib;
 
@@ -73,6 +17,8 @@ static VALUE cDO_Connection;
 static VALUE cDO_Command;
 static VALUE cDO_Result;
 static VALUE cDO_Reader;
+static VALUE cDO_Logger;
+static VALUE cDO_Logger_Message;
 
 static VALUE rb_cDate;
 static VALUE rb_cDateTime;
@@ -84,8 +30,6 @@ static VALUE cConnection;
 static VALUE cCommand;
 static VALUE cResult;
 static VALUE cReader;
-
-static VALUE eArgumentError;
 static VALUE eConnectionError;
 static VALUE eDataError;
 
@@ -120,28 +64,16 @@ static int jd_from_date(int year, int month, int day) {
   return (int) (floor(365.25 * (year + 4716)) + floor(30.6001 * (month + 1)) + day + b - 1524);
 }
 
-static void data_objects_debug(VALUE string, struct timeval* start) {
+static void data_objects_debug(VALUE connection, VALUE string, struct timeval* start) {
   struct timeval stop;
-  char *message;
+  VALUE message;
 
-  const char *query = rb_str_ptr_readonly(string);
-  size_t length     = rb_str_len(string);
-  char total_time[32];
-  do_int64 duration = 0;
+  gettimeofday(&stop, NULL);
+  do_int64 duration = (stop.tv_sec - start->tv_sec) * 1000000 + stop.tv_usec - start->tv_usec;
 
-  VALUE logger = rb_funcall(mSqlite3, ID_LOGGER, 0);
-  int log_level = NUM2INT(rb_funcall(logger, ID_LEVEL, 0));
+  message = rb_funcall(cDO_Logger_Message, ID_NEW, 3, string, rb_time_new(start->tv_sec, start->tv_usec), INT2NUM(duration));
 
-  if (0 == log_level) {
-    gettimeofday(&stop, NULL);
-
-    duration = (stop.tv_sec - start->tv_sec) * 1000000 + stop.tv_usec - start->tv_usec;
-
-    snprintf(total_time, 32, "%.6f", duration / 1000000.0);
-    message = (char *)calloc(length + strlen(total_time) + 4, sizeof(char));
-    snprintf(message, length + strlen(total_time) + 4, "(%s) %s", total_time, query);
-    rb_funcall(logger, ID_DEBUG, 1, rb_str_new(message, length + strlen(total_time) + 3));
-  }
+  rb_funcall(connection, ID_LOG, 1, message);
 }
 
 static void raise_error(VALUE self, sqlite3 *result, VALUE query) {
@@ -330,6 +262,12 @@ static VALUE typecast(sqlite3_stmt *stmt, int i, VALUE type, int encoding) {
     return ruby_value;
   }
 
+#ifdef HAVE_RUBY_ENCODING_H
+  rb_encoding * internal_encoding = rb_default_internal_encoding();
+#else
+  void * internal_encoding = NULL;
+#endif
+
   if(type == Qnil) {
     switch(original_type) {
       case SQLITE_INTEGER: {
@@ -354,7 +292,7 @@ static VALUE typecast(sqlite3_stmt *stmt, int i, VALUE type, int encoding) {
   if (type == rb_cInteger) {
     return LL2NUM(sqlite3_column_int64(stmt, i));
   } else if (type == rb_cString) {
-    return DO_STR_NEW((char*)sqlite3_column_text(stmt, i), length, encoding);
+    return DO_STR_NEW((char*)sqlite3_column_text(stmt, i), length, encoding, internal_encoding);
   } else if (type == rb_cFloat) {
     return rb_float_new(sqlite3_column_double(stmt, i));
   } else if (type == rb_cBigDecimal) {
@@ -371,12 +309,10 @@ static VALUE typecast(sqlite3_stmt *stmt, int i, VALUE type, int encoding) {
     return rb_funcall(rb_cByteArray, ID_NEW, 1, rb_str_new((char*)sqlite3_column_blob(stmt, i), length));
   } else if (type == rb_cClass) {
     return rb_funcall(mDO, rb_intern("full_const_get"), 1, rb_str_new((char*)sqlite3_column_text(stmt, i), length));
-  } else if (type == rb_cObject) {
-    return rb_marshal_load(rb_str_new((char*)sqlite3_column_text(stmt, i), length));
   } else if (type == rb_cNilClass) {
     return Qnil;
   } else {
-    return DO_STR_NEW((char*)sqlite3_column_text(stmt, i), length, encoding);
+    return DO_STR_NEW((char*)sqlite3_column_text(stmt, i), length, encoding, internal_encoding);
   }
 }
 
@@ -385,7 +321,7 @@ static VALUE typecast(sqlite3_stmt *stmt, int i, VALUE type, int encoding) {
 #define FLAG_PRESENT(query_values, flag) !NIL_P(rb_hash_aref(query_values, flag))
 
 static int flags_from_uri(VALUE uri) {
-  VALUE query_values = rb_funcall(uri, ID_QUERY, 0);
+  VALUE query_values = rb_funcall(uri, rb_intern("query"), 0);
 
   int flags = 0;
 
@@ -425,7 +361,7 @@ static VALUE cConnection_initialize(VALUE self, VALUE uri) {
   VALUE path;
   sqlite3 *db;
 
-  path = rb_funcall(uri, ID_PATH, 0);
+  path = rb_funcall(uri, rb_intern("path"), 0);
 
 #ifdef HAVE_SQLITE3_OPEN_V2
   ret = sqlite3_open_v2(rb_str_ptr_readonly(path), &db, flags_from_uri(uri), 0);
@@ -488,11 +424,11 @@ static VALUE cCommand_set_types(int argc, VALUE *argv, VALUE self) {
         if(TYPE(sub_entry) == T_CLASS) {
           rb_ary_push(type_strings, sub_entry);
         } else {
-          rb_raise(eArgumentError, "Invalid type given");
+          rb_raise(rb_eArgError, "Invalid type given");
         }
       }
     } else {
-      rb_raise(eArgumentError, "Invalid type given");
+      rb_raise(rb_eArgError, "Invalid type given");
     }
   }
 
@@ -570,7 +506,7 @@ static VALUE cCommand_execute_non_query(int argc, VALUE *argv, VALUE self) {
   if ( status != SQLITE_OK ) {
     raise_error(self, db, query);
   }
-  data_objects_debug(query, &start);
+  data_objects_debug(connection, query, &start);
 
   affected_rows = sqlite3_changes(db);
   insert_id = sqlite3_last_insert_rowid(db);
@@ -602,7 +538,7 @@ static VALUE cCommand_execute_reader(int argc, VALUE *argv, VALUE self) {
 
   gettimeofday(&start, NULL);
   status = sqlite3_prepare_v2(db, rb_str_ptr_readonly(query), -1, &sqlite3_reader, 0);
-  data_objects_debug(query, &start);
+  data_objects_debug(connection, query, &start);
 
   if ( status != SQLITE_OK ) {
     raise_error(self, db, query);
@@ -624,7 +560,7 @@ static VALUE cCommand_execute_reader(int argc, VALUE *argv, VALUE self) {
     // Whoops...  wrong number of types passed to set_types.  Close the reader and raise
     // and error
     rb_funcall(reader, rb_intern("close"), 0);
-    rb_raise(eArgumentError, "Field-count mismatch. Expected %ld fields, but the query yielded %d", RARRAY_LEN(field_types), field_count);
+    rb_raise(rb_eArgError, "Field-count mismatch. Expected %ld fields, but the query yielded %d", RARRAY_LEN(field_types), field_count);
   }
 
   for ( i = 0; i < field_count; i++ ) {
@@ -720,13 +656,14 @@ static VALUE cReader_field_count(VALUE self) {
 void Init_do_sqlite3() {
   rb_require("bigdecimal");
   rb_require("date");
+  rb_require("data_objects");
+
+  ID_CONST_GET = rb_intern("const_get");
 
   // Get references classes needed for Date/Time parsing
-  rb_cDate = RUBY_CLASS("Date");
-  rb_cDateTime = RUBY_CLASS( "DateTime");
-  rb_cBigDecimal = RUBY_CLASS("BigDecimal");
-
-  rb_funcall(rb_mKernel, rb_intern("require"), 1, rb_str_new2("data_objects"));
+  rb_cDate = CONST_GET(rb_mKernel, "Date");
+  rb_cDateTime = CONST_GET(rb_mKernel, "DateTime");
+  rb_cBigDecimal = CONST_GET(rb_mKernel, "BigDecimal");
 
 #ifdef RUBY_LESS_THAN_186
   ID_NEW_DATE = rb_intern("new0");
@@ -734,9 +671,9 @@ void Init_do_sqlite3() {
   ID_NEW_DATE = rb_intern("new!");
 #endif
   ID_RATIONAL = rb_intern("Rational");
-  ID_LOGGER = rb_intern("logger");
-  ID_DEBUG = rb_intern("debug");
-  ID_LEVEL = rb_intern("level");
+  ID_NEW = rb_intern("new");
+  ID_ESCAPE = rb_intern("escape_sql");
+  ID_LOG = rb_intern("log");
 
   // Get references to the Extlib module
   mExtlib = CONST_GET(rb_mKernel, "Extlib");
@@ -749,15 +686,16 @@ void Init_do_sqlite3() {
   cDO_Command = CONST_GET(mDO, "Command");
   cDO_Result = CONST_GET(mDO, "Result");
   cDO_Reader = CONST_GET(mDO, "Reader");
+  cDO_Logger = CONST_GET(mDO, "Logger");
+  cDO_Logger_Message = CONST_GET(cDO_Logger, "Message");
 
   // Initialize the DataObjects::Sqlite3 module, and define its classes
   mSqlite3 = rb_define_module_under(mDO, "Sqlite3");
 
-  eArgumentError = CONST_GET(rb_mKernel, "ArgumentError");
   eConnectionError = CONST_GET(mDO, "ConnectionError");
   eDataError = CONST_GET(mDO, "DataError");
 
-  cConnection = SQLITE3_CLASS("Connection", cDO_Connection);
+  cConnection = DRIVER_CLASS("Connection", cDO_Connection);
   rb_define_method(cConnection, "initialize", cConnection_initialize, 1);
   rb_define_method(cConnection, "dispose", cConnection_dispose, 0);
   rb_define_method(cConnection, "quote_boolean", cConnection_quote_boolean, 1);
@@ -765,20 +703,41 @@ void Init_do_sqlite3() {
   rb_define_method(cConnection, "quote_byte_array", cConnection_quote_byte_array, 1);
   rb_define_method(cConnection, "character_set", cConnection_character_set, 0);
 
-  cCommand = SQLITE3_CLASS("Command", cDO_Command);
+  cCommand = DRIVER_CLASS("Command", cDO_Command);
   rb_define_method(cCommand, "set_types", cCommand_set_types, -1);
   rb_define_method(cCommand, "execute_non_query", cCommand_execute_non_query, -1);
   rb_define_method(cCommand, "execute_reader", cCommand_execute_reader, -1);
 
-  cResult = SQLITE3_CLASS("Result", cDO_Result);
+  cResult = DRIVER_CLASS("Result", cDO_Result);
 
-  cReader = SQLITE3_CLASS("Reader", cDO_Reader);
+  cReader = DRIVER_CLASS("Reader", cDO_Reader);
   rb_define_method(cReader, "close", cReader_close, 0);
   rb_define_method(cReader, "next!", cReader_next, 0);
   rb_define_method(cReader, "values", cReader_values, 0);
   rb_define_method(cReader, "fields", cReader_fields, 0);
   rb_define_method(cReader, "field_count", cReader_field_count, 0);
 
+  rb_global_variable(&ID_NEW_DATE);
+  rb_global_variable(&ID_RATIONAL);
+  rb_global_variable(&ID_CONST_GET);
+  rb_global_variable(&ID_ESCAPE);
+  rb_global_variable(&ID_LOG);
+  rb_global_variable(&ID_NEW);
+
+  rb_global_variable(&rb_cDate);
+  rb_global_variable(&rb_cDateTime);
+  rb_global_variable(&rb_cBigDecimal);
+  rb_global_variable(&rb_cByteArray);
+
+  rb_global_variable(&mDO);
+  rb_global_variable(&cDO_Logger_Message);
+
+  rb_global_variable(&cResult);
+  rb_global_variable(&cReader);
+
+  rb_global_variable(&eConnectionError);
+  rb_global_variable(&eDataError);
+
   OPEN_FLAG_READONLY = rb_str_new2("read_only");
   rb_global_variable(&OPEN_FLAG_READONLY);
   OPEN_FLAG_READWRITE = rb_str_new2("read_write");
@@ -790,4 +749,5 @@ void Init_do_sqlite3() {
   OPEN_FLAG_FULL_MUTEX = rb_str_new2("full_mutex");
   rb_global_variable(&OPEN_FLAG_FULL_MUTEX);
 
+  Init_do_sqlite3_extension();
 }
